#include "board_led.h"
#include <avr/io.h>
#include <stdint.h>
#include <util/delay.h>

void uart_init(uint32_t baudrate) {
  uint16_t ubbr = F_CPU / 8 / baudrate - 1;

  // Définit la vitesse de transmission
  UBRR0H = (uint8_t)(ubbr >> 8);
  UBRR0L = (uint8_t)ubbr;

  // Active le doublement de la vitesse de transmission
  UCSR0A = (1 << U2X0);
  // Active la réception et la transmission
  UCSR0B = (1 << RXEN0) | (1 << TXEN0);
  // 1 stop bit, 8 data bits
  UCSR0C = (1 << USBS0) | (3 << UCSZ00);
}

void uart_send_byte(uint8_t data) {
  // Wait for empty transmit buffer
  while (!(UCSR0A & (1 << UDRE0)))
    ;
  // Put data into buffer, sends the data
  UDR0 = data;
}

void uart_send_string(char *str) {
  while (*str) {
    uart_send_byte(*str);
    str++;
  }
}

uint8_t uart_receive() {
  // Wait for data to be received
  while (!(UCSR0A & (1 << RXC0)))
    ;
  // Get and return received data from buffer
  return UDR0;
}

void uart_receive_string(char *buffer, uint8_t max_size) {
  uint8_t i = 0;
  while (i < max_size - 1) {
    buffer[i] = uart_receive();
    if (buffer[i] == '\r' || buffer[i] == '\n') {
      break;
    }
    i++;
  }
  buffer[i] = '\0';
}

int main() {
  uart_init(115200);
  board_led_init();

  char buffer[64];

  while (1) {
    uart_send_string("Enter a string: ");
    uart_receive_string(buffer, sizeof(buffer));

    board_led_set(1);
    uart_send_string("\r\nYou entered: ");
    uart_send_string(buffer);
    uart_send_string("\r\n");
    board_led_set(0);
  }
}
